*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE QUADQD (NDIM,NUMERO,NBPTQ,POIDQ,COORQ,DEXACT)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Fevrier 1988)
* Derniere modification : D.Martin (06 juillet 2005)
* Version 2
*
* Formules de quadrature sur le quadrangle [0.,1.] x [0.,1.]
* (programme pour que les valeurs des coeff. soit corrects en double precision
*  pour les compilations implicites en double precision)
*
*-- Variables d'entree -- 
* NDIM   dimension d'espace
* NUMERO numero du schema de quadrature (convention)
*       (par convention = degre de precision pour les formules de Gauss)
*-- Variables en sortie --
* NBPTQ  nombre de points du schema de quadrature
* POIDQ  tableau des poids du schema de quadrature
* COORQ  tableau des coordonnees des points de quadrature
* DEXACT degre de precision de la formule (pour impression)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      INTEGER NDIM,NUMERO,NBPTQ,DEXACT
      REAL    POIDQ(*),COORQ(NDIM,*)
*
      INTEGER I,J,NSAUT,NSAUTI,NSAUTJ,NUPTQ,NBPTQ1
      REAL    X,P1,P2,PM
      INTEGER     DEGMAX
      PARAMETER ( DEGMAX = 99 )
*-----------------------------------------------------------------------
      IF (NUMERO.EQ.-1) THEN
*
*        Formule 'nodale' utilisant les noeuds de l'element l'element Q1
*        exacte pour Q1 (Formule du trapeze tensorielle)
*
         DEXACT=1
         NBPTQ=4
         CALL QULKCO (ABS(NUMERO),NDIM,COORQ)
         P1=1./4
         DO 1001 I=1,NBPTQ
            POIDQ(I)=P1
1001     CONTINUE
*
      ELSEIF (NUMERO.EQ.-2) THEN
*
*        Formule 'nodale' utilisant les noeuds de l'element l'element Q2
*        exacte pour Q3 (Formule de Simpson tensorielle)
*
         DEXACT=3
         NBPTQ=9
         CALL QULKCO (ABS(NUMERO),NDIM,COORQ)
         P1=1./36.
         P2=1./9.
         DO 1002 I=1,4
            POIDQ(I)  =P1
            POIDQ(I+4)=P2
1002     CONTINUE
         POIDQ(9)=4./9.
*
      ELSEIF (NUMERO.EQ.1) THEN
*
*        Formule de GAUSS a 1 point exacte pour Q1([0.,1.]**2)  
*
         DEXACT=1
         DO 10 I=1,NDIM
            COORQ(1,I)=.5
10       CONTINUE
         POIDQ(1)=1.
         NBPTQ=1
*
      ELSEIF (NUMERO.EQ.3) THEN
*
*        *-----*-----*
*        |  2     4  |  Formule de GAUSS a 4 points exacte
*        |           |  pour Q3([0.,1.]**2) obtenue par
*        *           *  'Carre tensoriel' de la formule 1-D a 2 points
*        |           |
*        |  1     3  |
*        *-----*-----*
         NSAUT=1+(NUMERO+1)/2
         CALL SEGMQD (NDIM,NUMERO,NBPTQ,POIDQ,COORQ,NSAUT,DEXACT)
         NUPTQ=0
         NSAUTI=1
         DO 31 I=1,NBPTQ
            X=COORQ(1,NSAUTI)
            NSAUTJ=1
            DO 30 J=1,NBPTQ
               NUPTQ=NUPTQ+1
               COORQ(1,NUPTQ)=X
               COORQ(2,NUPTQ)=COORQ(1,NSAUTJ)
               NSAUTJ=NSAUTJ+NSAUT
30          CONTINUE
            NSAUTI=NSAUTI+NSAUT
31       CONTINUE
         NBPTQ=NUPTQ
         P1=POIDQ(1)*POIDQ(1)
         POIDQ(1)=P1
         POIDQ(2)=P1
         POIDQ(3)=P1
         POIDQ(4)=P1
         NBPTQ=NUPTQ
*  
      ELSEIF (NUMERO.EQ.5) THEN
*
*        *-----*-----*
*        | 3   6   9 |  Formule de GAUSS a 9 points exacte
*        |           |  pour Q5([0.,1.]**2) obtenue par
*        * 2   5   8 *  'Carre tensoriel' de la formule 1-D a 3 points
*        |           |
*        | 1   4   7 |
*        *-----*-----*
         NSAUT=1+(NUMERO+1)/2
         CALL SEGMQD (NDIM,NUMERO,NBPTQ,POIDQ,COORQ,NSAUT,DEXACT)
         NUPTQ=0
         NSAUTI=1
         DO 51 I=1,NBPTQ
            X=COORQ(1,NSAUTI) 
            NSAUTJ=1
            DO 50 J=1,NBPTQ
               NUPTQ=NUPTQ+1  
               COORQ(1,NUPTQ)=X
               COORQ(2,NUPTQ)=COORQ(1,NSAUTJ)
               NSAUTJ=NSAUTJ+NSAUT
50          CONTINUE
            NSAUTI=NSAUTI+NSAUT
51       CONTINUE
         NBPTQ=NUPTQ
         P1=POIDQ(1)*POIDQ(1)
         P2=POIDQ(1)*POIDQ(2)
         PM=POIDQ(2)*POIDQ(2)
         POIDQ(1)=P1
         POIDQ(2)=P2
         POIDQ(3)=P1
         POIDQ(4)=P2
         POIDQ(5)=PM
         POIDQ(6)=P2
         POIDQ(7)=P1
         POIDQ(8)=P2
         POIDQ(9)=P1
*
      ELSEIF (MOD(NUMERO,2).EQ.1.AND.NUMERO.LE.DEGMAX) THEN
*
*        Formule de GAUSS a (k+1)^2 points exacte pour Q_{2k-1}([0.,1.]**2)
*        obtenue par 'Carre tensoriel' de la formule 1-D a k+1 points
*        avec k = (NUMERO+1)/2
*
         NSAUT=1+(NUMERO+1)/2
         CALL SEGMQD (NDIM,NUMERO,NBPTQ1,POIDQ,COORQ,NSAUT,DEXACT)
         NUPTQ=0
         NSAUTI=1
         DO 111 I=1,NBPTQ1
            X=COORQ(1,NSAUTI)
            NSAUTJ=1
            DO 110 J=1,NBPTQ1
               NUPTQ=NUPTQ+1
               COORQ(1,NUPTQ)=X
               COORQ(2,NUPTQ)=COORQ(1,NSAUTJ)
               NSAUTJ=NSAUTJ+NSAUT
110         CONTINUE
            NSAUTI=NSAUTI+NSAUT
111      CONTINUE
         NBPTQ=NUPTQ
         DO 115 I=NBPTQ1,1,-1
            X=POIDQ(I)
            DO 114 J=NBPTQ1,1,-1
               POIDQ(NUPTQ)=X*POIDQ(J)
               NUPTQ=NUPTQ-1
114         CONTINUE
115      CONTINUE
*
      ELSE
*
         CALL QUXXQD (NDIM,NUMERO,NBPTQ,POIDQ,COORQ,DEXACT)
      ENDIF
                                                                    END !QuadQd
