*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE SE105B (KELFCT,XSP,P,DP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteurs : D.Martin & G.Vial (Septembre 2000)
* Derniere modification : D.Martin (12 juillet 2005)
* Version 2
*
* Fonctions de base et derivees sur le segment de Lagrange d'ordre 5
* pour l'interpolation de Lagrange aux points de Gauss-Lobatto
*
*-- Arguments d'entree --
* KELFCT variable CHARACTER indiquant
*        si KELFCT(1:1).NE.'N' Calcul des fonctions de base
*        si KELFCT(2:2).NE.'N' Calcul des derivees premieres des fct. de base
*        si KELFCT(3:3).NE.'N' Calcul des derivees secondes des fct. de base
* X      coordonnees du point ou l'on demande la calcul
*-- Arguments de sortie --
* P     tableau contenant les valeurs des fonctions au point X
* DP        -       -            -     -  derivees premieres
* DDP       -       -            -     -  derivees secondes
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*) KELFCT
      REAL    XSP,P(*),DP(*)
*
      DOUBLE PRECISION X,XM1,XXM1,A,CGALO6,CGALO5
*-----------------------------------------------------------------------
*        Element de Lagrange P5 aux 6 points de Gauss-Lobatto
*        ranges par abcsisses croissantes selon l'ordre 2 6 5 4 3 1
*-----------------------------------------------------------------------
      X = DBLE(XSP)
      XM1 = X-1.
      XXM1=X*XM1
      CGALO6 = .11747233803526765357D0
      CGALO5 = .35738424175967745184D0
*     Fonctions de base
      IF (KELFCT(1:1).NE.'N') THEN
         A=42*(X-CGALO6)*(XM1+CGALO6)*(X-CGALO5)*(XM1+CGALO5)
         P(1)= A*X
         P(2)=-A*XM1
         
         A=100.07221064631794722D0*XXM1*(X-CGALO5)*(XM1+CGALO5)
         P(3)=-A*(X-CGALO6)
         P(6)= A*(XM1+CGALO6)
         
         A=121.16745708464368404D0*XXM1*(X-CGALO6)*(XM1+CGALO6)
         P(4)= A*(X-CGALO5)
         P(5)=-A*(XM1+CGALO5)
      ENDIF
*     Derivees premieres
      IF (KELFCT(2:2).NE.'N') THEN
         DP(1)= 210*(X-.48500549446997329292D-1)
     &        *(X-.23860073755186230508D0)*(X-.5170472951043675023D0)
     &        *(X-.79585141789677286336D0)
         DP(2)=-210*(X-.20414858210322713671D0)
     &        *(X-.48295270489563249746D0)*(X-.76139926244813769531D0)
     &        *(X-.9514994505530026706D0)
     
         DP(3)=-500.3610532315897361D0*(XM1+CGALO6)
     &        *(X-.48728826750077161755D-1)*(X-.24005783550119095217D0)
     &        *(X-.52266354621221366255D0)
         DP(6)= 500.3610532315897361D0*(X-CGALO6)
     &        *(X-.47733645378778633743D0)*(X-.75994216449880904816D0)
     &        *(X-.95127117324992283797D0)
     
         DP(4)= 605.8372854232184202D0*(XM1+CGALO5)
     &        *(X-.49483788070174570583D-1)*(X-.24581621016878540174D0)
     &        *(X-.947991636928459441D0)
         DP(5)=-605.8372854232184202D0*(X-CGALO5)
     &        *(X-.52008363071540559019D-1)*(X-.7541837898312145981D0)
     &        *(X-.95051621192982542954D0)
      ENDIF
*-----------------------------------------------------------------------
      END
