*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE SEGLKB (K,KELFCT,XSP,P,DP,DDP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteurs : D.Martin & G.Vial (Septembre 2000)
* Derniere modification : D.Martin (12 juillet 2005)
* Version 2
*
* Fonctions de base et derivees sur le segment de Lagrange d'ordre k
* pour l'interpolation de Lagrange aux points de Gauss-Lobatto
* pour 2 < k < 21 et k = 24, 32, 40
*
*-- Arguments d'entree --
* K      ordre de l'element de Lagrange
* KELFCT variable CHARACTER indiquant
*        si KELFCT(1:1).NE.'N' Calcul des fonctions de base
*        si KELFCT(2:2).NE.'N' Calcul des derivees premieres des fct. de base
*        si KELFCT(3:3).NE.'N' Calcul des derivees secondes des fct. de base
* X      coordonnees du point ou l'on demande la calcul
*-- Arguments de sortie --
* P     tableau contenant les valeurs des fonctions au point X
* DP        -       -            -     -  derivees premieres
* DDP       -       -            -     -  derivees secondes
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*) KELFCT
      INTEGER K
      REAL    XSP,P(*),DP(*),DDP(*)
*
      CHARACTER*4 ERRSTR
      DOUBLE PRECISION X,XM1,XXM1,XMHALF,CGALOB,FACT,A
*-----------------------------------------------------------------------
      X=DBLE(XSP)
      XM1=X-1.D0
      XXM1=X*XM1
*
      GOTO (3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20) K-2
      GOTO 21
*
*        Element de Lagrange P3 aux 4 points de Gauss-Lobatto
*        ranges par abcsisses croissantes selon l'ordre 2 4 3 1
*-----------------------------------------------------------------------
 3    CGALOB = .27639320225002103036D0
*     Fonctions de base
      IF (KELFCT(1:1).NE.'N') THEN
*        CGALOB*(1-CGALOB) = 1/5
         A=5.D0*(X-CGALOB)*(XM1+CGALOB)
         P(1)= A*X
         P(2)=-A*XM1

         A=11.180339887498948482D0*XXM1
         P(3)=-A*(X-CGALOB)
         P(4)= A*(XM1+CGALOB)
      ENDIF
*     Derivees premieres
      IF (KELFCT(2:2).NE.'N') THEN
         DP(1)= 15*(X-.12251482265544138D0)*(XM1+.45584815598877471D0)
         DP(2)=-15*(XM1+.12251482265544138D0)*(X-.45584815598877471D0)

         A = 3.*11.180339887498948482D0
         DP(3)=-A*(X-.1273220037500350506D0)*(XM1+CGALOB)
         DP(4)= A*(XM1+.1273220037500350506D0)*(X-CGALOB)
      ENDIF
      RETURN
 
*        Element de Lagrange P4 aux 5 points de Gauss-Lobatto
*        ranges par abcsisses croissantes selon l'ordre 2 5 4 3 1
*-----------------------------------------------------------------------
 4    CGALOB = .172673164646011428D0
      XMHALF=X-.5D0
*     Fonctions de base
      IF (KELFCT(1:1).NE.'N') THEN
*        CGALOB*(1-CGALOB)*.5 = 1/14
         A=14*(X-CGALOB)*(XM1+CGALOB)
         P(1)= A*X*XMHALF
         P(2)= A*XMHALF*XM1
         P(4)= (8D0/3)*A*XXM1
         
         A=(98D0/3)*XXM1*XMHALF
         P(3)=-A*(X-CGALOB)
         P(5)=-A*(XM1+CGALOB)
      ENDIF
*     Derivees premieres
      IF (KELFCT(2:2).NE.'N') THEN
         DP(1)=56*(X-.70500220988849838312D0)
     &        *(X-.34700376603835188472D0)*(X-.72994024073149732155D-1)
         DP(2)=56*(X-.29499779011150161692D0)
     &        *(X-.65299623396164811504D0)*(X-.92700597592685026804D0)
         DP(4)= (448D0/3)*XMHALF
     &        *(X-.77422872635741711245D-1)*(X-.92257712736425828881D0)

         DP(3)=-(392D0/3)*(XM1+CGALOB)
     &        *(X-.35334326765899685556D0)*(X-.73834770471523143566D-1)
         DP(5)=-(392D0/3)*(X-CGALOB)  
     &        *(X-.64665673234100314446D0)*(X-.92616522952847685626D0)
      ENDIF
      RETURN

*     Elements de Lagrange P5 a P20 aux points de Gauss-Lobatto
*-----------------------------------------------------------------------
  5   CALL SE105B (KELFCT,XSP,P,DP)
      RETURN
  6   CALL SE106B (KELFCT,XSP,P,DP)
      RETURN
  7   CALL SE107B (KELFCT,XSP,P,DP)
      RETURN
  8   CALL SE108B (KELFCT,XSP,P,DP)
      RETURN
  9   CALL SE109B (KELFCT,XSP,P,DP)
      RETURN
 10   CALL SE110B (KELFCT,XSP,P,DP)
      RETURN
 11   CALL SE111B (KELFCT,XSP,P,DP)
      RETURN
 12   CALL SE112B (KELFCT,XSP,P,DP)
      RETURN
 13   CALL SE113B (KELFCT,XSP,P,DP)
      RETURN
 14   CALL SE114B (KELFCT,XSP,P,DP)
      RETURN
 15   CALL SE115B (KELFCT,XSP,P,DP)
      RETURN
 16   CALL SE116B (KELFCT,XSP,P,DP)
      RETURN
 17   CALL SE117B (KELFCT,XSP,P,DP)
      RETURN
 18   CALL SE118B (KELFCT,XSP,P,DP)
      RETURN
 19   CALL SE119B (KELFCT,XSP,P,DP)
      RETURN
 20   CALL SE120B (KELFCT,XSP,P,DP)
      RETURN
      
*     Elements de Lagrange P24, P32, P40 aux points de Gauss-Lobatto
*-----------------------------------------------------------------------
 21   IF (K.EQ.24) THEN
         CALL SE124B (KELFCT,XSP,P,DP)
      ELSEIF (K.EQ.32) THEN
         CALL SE132B (KELFCT,XSP,P,DP)
      ELSEIF (K.EQ.40) THEN
         CALL SE140B (KELFCT,XSP,P,DP)
      ELSE
         CALL ENCLER (K,ERRSTR)
         CALL BAISE  ('Interpolation de Lagrange Gauss-Lobatto '
     &              //'non prevue pour K = '//ERRSTR//' .')
      ENDIF
      END
